@@ -3,14 +3,14 @@
 #'
 #' Performs a compromise power analysis, i.e. determines the critical chi-square along with the implied alpha and beta, given a specified alpha/beta ratio, effect, N, and df
 #'
-#' @param effect effect size specifying the discrepancy between H0 and H1
+#' @param effect effect size specifying the discrepancy between H0 and H1  (a list for multiple group models)
 #' @param effect.measure type of effect, one of "F0","RMSEA", "Mc", "GFI", AGFI"
 #' @param abratio the ratio of alpha to beta
-#' @param N the number of observations
+#' @param N the number of observations  (a list for multiple group models)
 #' @param df the model degrees of freedom
 #' @param p the number of observed variables, required for effect.measure = "GammaHat", "GFI",  and "AGFI"
-#' @param SigmaHat model implied covariance matrix. Use in conjuntion with Sigma to define effect and effect.measure. 
-#' @param Sigma population covariance matrix. Use in conjuntion with SigmaHat to define effect and effect.measure.
+#' @param SigmaHat model implied covariance matrix (a list for multiple group models). Use in conjuntion with Sigma to define effect and effect.measure.  
+#' @param Sigma population covariance matrix (a list for multiple group models). Use in conjuntion with SigmaHat to define effect and effect.measure.
 #' @return list
 #' @examples
 #' \dontrun{
@@ -26,6 +26,10 @@ semPower.compromise  <- function(effect = NULL, effect.measure = NULL,
 
   if(!is.null(effect.measure)) effect.measure <- toupper(effect.measure)
   
+  # convert vectors to lists
+  if(!is.list(effect) && length(effect) > 1) effect <- as.list(effect)
+  if(!is.list(N) && length(N) > 1) N <- as.list(N) 
+  
   validateInput('compromise', effect = effect, effect.measure = effect.measure,
                 alpha = NULL, beta = NULL, power = NULL, abratio = abratio,
                 N = N, df = df, p = p,
@@ -34,16 +38,41 @@ semPower.compromise  <- function(effect = NULL, effect.measure = NULL,
 
   if(!is.null(SigmaHat)){ # sufficient to check for on NULL matrix; primary validity check is in validateInput
     effect.measure <- 'F0'
-    p <- ncol(SigmaHat)
+    p <- ifelse(is.list(SigmaHat), ncol(SigmaHat[[1]]), ncol(SigmaHat))
   }
 
-  fmin <- getF(effect, effect.measure, df, p, SigmaHat, Sigma)
-  fit <- getIndices.F(fmin, df, p, SigmaHat, Sigma)
-  ncp <- getNCP(fmin, N)
+  # make sure N/effects have the same length
+  if((is.list(effect) || is.list(SigmaHat)) && length(N) == 1){
+    N <- as.list(rep(N, ifelse(is.null(SigmaHat), length(effect), length(SigmaHat))))
+  }
+  if(is.null(SigmaHat) && is.list(N) && length(effect) == 1){
+    effect <- as.list(rep(effect, length(N)))
+  }
+  ngroups <- ifelse(is.null(N), 1, length(N))
+  
+  # obsolete, single group case only
+  # fmin <- getF(effect, effect.measure, df, p, SigmaHat, Sigma)
+  # fit <- getIndices.F(fmin, df, p, SigmaHat, Sigma)
+  
+  if(!is.null(effect)){
+    fmin.g <- sapply(effect, FUN = getF, effect.measure = effect.measure, df = df, p = p)
+  }
+  if(!is.null(SigmaHat)){
+    if(is.list(Sigma)){
+      fmin.g <- sapply(seq_along(SigmaHat), FUN = function(x) {getF.Sigma(SigmaHat = SigmaHat[[x]], S = Sigma[[x]]) })
+    }else{
+      fmin.g <- getF.Sigma(SigmaHat = SigmaHat, S = Sigma)
+    }
+  }
+  
+  fmin <- sum(unlist(fmin.g) * unlist(N) / sum(unlist(N)))
+  fit <- getIndices.F(fmin, df, p, SigmaHat, Sigma, N)
+  
+  ncp <- getNCP(fmin.g, N)
   log.abratio <- log(abratio)
 
   if(ncp >= 1e5)
-    warning('ncp is larger than 1e5, this is going to cause trouble')
+    warning('NCP is larger than 100000, this is going to cause trouble.')
 
 
   # determine max/min chi for valid alpha/beta prob
@@ -51,7 +80,7 @@ semPower.compromise  <- function(effect = NULL, effect.measure = NULL,
   # central chi always gives reusult up to 1e-320
   max <- qchisq(log(1e-320), df, lower.tail = F, log.p = T)
 
-  # non-central chi accuracy is usaually lower, depending on df and ncp
+  # non-central chi accuracy is usually lower, depending on df and ncp
   pmin <- -Inf
   testp <- 1e-320
   while(is.infinite(pmin)){
